import adapay
from adapay.api import urls
from adapay.api.api_request import ApiRequest


class Member(object):

    @classmethod
    def create(cls, **kwargs):
        """
        创建用户
        """
        url = adapay.base_url + urls.member_create
        return ApiRequest.post(url, kwargs)

    @classmethod
    def query(cls, **kwargs):
        """
        查询用户
        """
        url = adapay.base_url + urls.member_query.format(member_id=kwargs.get('member_id'))
        return ApiRequest.get(url, kwargs)

    @classmethod
    def query_list(cls, **kwargs):
        """
        查询用户
        """
        url = adapay.base_url + urls.member_query_list
        return ApiRequest.get(url, kwargs)

    @classmethod
    def update(cls, **kwargs):
        """
        更新用户
        """
        url = adapay.base_url + urls.member_update
        return ApiRequest.post(url, kwargs)





"""
2019.8.1 create by jun.hu
默认请求对象
"""
import json

import requests

import adapay
from adapay.utils.log_util import log_error, log_info
from adapay.utils.common_utils import get_plain_text, pop_empty_value
from adapay.utils.pycrypt_utils import rsa_sign, rsa_design


class ApiRequest:

    @staticmethod
    def post(url, request_params, request_file=None):
        return ApiRequest._request(url, 'post', request_params, request_file)

    @staticmethod
    def get(url, request_params):
        return ApiRequest._request(url, 'get', request_params)

    @staticmethod
    def _build_request_info(url, method, params, files):
        """
        根据请求方式构造请求头和请求参数
        :return: header 请求头 params 请求参数
        """
        header = {'authorization': adapay.api_key,
                  "sdk_version": 'python_v' + adapay.__version__,
                  'signature': ''}

        params = pop_empty_value(params)

        plain_text = url

        if params:
            if 'post' == method and not files:
                plain_text = plain_text + json.dumps(params)
            else:
                plain_text = plain_text + get_plain_text(params)

        if not adapay.private_key:
            raise RuntimeError('privite_key is none')

        flag, cipher_text = rsa_sign(adapay.private_key, plain_text, 'utf-8')

        if not flag:
            log_error('request to {}, sign error {} '.format(url, cipher_text))

        header.update({'signature': cipher_text})
        log_info('request to {}, param is {}, \nhead is {}'.format(url, params, header))

        return header, params

    @staticmethod
    def _request(url, method, params=None, files=None):
        """
        执行请求
        :param url: 请求地址
        :param method: 请求方法类型
        :param params: 请求参数
        :param files: 上传的文件
        :return: 网路请求返回的数据
        """
        header, params = ApiRequest._build_request_info(url, method, params, files)

        http_method = getattr(requests, method or 'post')

        if files:
            resp = http_method(url, data=params, files=files, timeout=adapay.connect_timeout, headers=header)

        elif method == 'post':
            resp = http_method(url, json=params, files=files, timeout=adapay.connect_timeout, headers=header)

        else:
            resp = http_method(url, params, timeout=adapay.connect_timeout, headers=header)

        log_info('request to {}, resp is {}'.format(url, resp.text))

        return ApiRequest._build_return_data(resp)

    @staticmethod
    def _build_return_data(resp):

        try:
            resp_json = json.loads(resp.text)
        except Exception as e:
            log_error('adapay resp_code is ' + str(resp.status_code))
            log_error(str(e))
            return resp.text

        data = resp_json.get('data', '')
        resp_sign = resp_json.get('signature', '')

        if not resp_sign:
            return resp_json

        if not adapay.public_key:
            raise RuntimeError('public_key is none')

        flag, info = rsa_design(resp_sign, data, adapay.public_key)

        if not flag:
            log_error('check signature error !'.format(info))
            raise RuntimeError(info)

        return json.loads(data)



"""
2019.8.1 create by jun.hu
退款接口
"""

import adapay
from adapay.api.api_request import ApiRequest
from adapay.api.urls import refund_create, refund_query


class Refund(object):

    @classmethod
    def create(cls, **kwargs):
        """
        发起退款流程
        """

        url = adapay.base_url + refund_create.format(kwargs.get('payment_id', ''))
        return ApiRequest.post(url, kwargs)

    @classmethod
    def query(cls, **kwargs):
        """
        退款流程查询
        """

        return ApiRequest.get(adapay.base_url + refund_query, kwargs)





import logging
import adapay
from fishbase.fish_logger import logger


def log_info(info):
    if logging.INFO == adapay.log_level:
        logger.info(adapay.log_tag + '{}'.format(info))


def log_warning(warning_info):
    if adapay.log_level in [logging.INFO, logging.WARNING]:
        logger.warning(adapay.log_tag + '{}'.format(warning_info))


def log_error(error_info):
    if adapay.log_level in [logging.INFO, logging.WARNING, logging.ERROR]:
        logger.error(adapay.log_tag + '{}'.format(error_info))



import adapay
from adapay.api import urls
from adapay.api.api_request import ApiRequest


class Account(object):

    @classmethod
    def create_settle(cls, **kwargs):
        """
        创建结算账户
        """
        url = adapay.base_url + urls.settle_account_create
        return ApiRequest.post(url, kwargs)

    @classmethod
    def query_settle(cls, **kwargs):
        """
        查询结算账户
        """
        url = adapay.base_url + \
              urls.settle_account_query.format(settle_account_id=kwargs.get('settle_account_id'))
        return ApiRequest.get(url, kwargs)

    @classmethod
    def modify_settle(cls, **kwargs):
        """
        修改结算配置
        """
        url = adapay.base_url + urls.settle_account_modify
        return ApiRequest.post(url, kwargs)

    @classmethod
    def delete_settle(cls, **kwargs):
        """
        删除结算账户
        """
        url = adapay.base_url + urls.settle_account_delete
        return ApiRequest.post(url, kwargs)

    @classmethod
    def query_settle_details(cls, **kwargs):
        """
        查询结算账户明细
        """
        url = adapay.base_url + urls.settle_account_detail_query
        return ApiRequest.get(url, kwargs)





import os

import adapay
from adapay.api import urls
from adapay.api.api_request import ApiRequest


class CorpMember(object):

    @classmethod
    def create(cls, **kwargs):
        """
        创建企业用户
        """
        url = adapay.base_url + urls.corp_member_create
        file_path = kwargs.get('attach_file')
        files = {'attach_file': (os.path.basename(file_path), open(file_path, 'rb'), 'application/octet-stream')}
        kwargs.pop('attach_file')
        return ApiRequest.post(url, kwargs, files)

    @classmethod
    def query(cls, **kwargs):
        """
        查询企业用户
        """
        url = adapay.base_url + urls.corp_member_query.format(member_id=kwargs.get('member_id'))
        return ApiRequest.get(url, kwargs)





#!/usr/bin/env python
import json
import threading
import uuid

from paho.mqtt import client as mqtt

import adapay
from adapay.api.api_request import ApiRequest
from adapay.api.urls import pay_message_token
from adapay.utils.log_util import log_info, log_error


class AdapayMessage:
    instance_id = ''
    access_key = ''
    group_id = ''
    client_id = ''
    topic = ''
    broker_url = ''

    connect_callback = None
    subscribe_callback = None
    message_received = None

    @staticmethod
    def init():
        AdapayMessage.instance_id = 'post-cn-0pp18zowf0m'

        AdapayMessage.access_key = 'LTAIOP5RkeiuXieW'

        AdapayMessage.group_id = 'GID_CRHS_ASYN'

        AdapayMessage.client_id = AdapayMessage.group_id + '@@@' + str(
            hash(adapay.api_key + (str(uuid.uuid1()))))

        AdapayMessage.topic = 'topic_crhs_sender/' + adapay.api_key

        AdapayMessage.broker_url = 'post-cn-0pp18zowf0m.mqtt.aliyuncs.com'

    @staticmethod
    def _on_connect(client, userdata, flags, resp_code):
        """
        建立长连接成功回调
        """
        log_info('connected with result code ' + str(resp_code))

        if AdapayMessage.connect_callback is not None:
            AdapayMessage.connect_callback(resp_code)

        if resp_code == 0:
            client.subscribe(AdapayMessage.topic, 0)

    @staticmethod
    def _on_disconnect(client, userdata, resp_code):
        """
        :param resp_code:
         长连接链接失败回调
        1	伪造 Token，不可解析
        2	Token 已经过期
        3	Token 已经被吊销
        4	资源和 Token 不匹配
        5	权限类型和 Token 不匹配
        8	签名不合法
        -1	帐号权限不合法

        :return:
        """
        log_info('unexpected disconnection %s' % resp_code)

        if AdapayMessage.connect_callback is not None:
            AdapayMessage.connect_callback(resp_code)

    @staticmethod
    def _on_subscribe(client, userdata, mid, granted_qos):
        """
        订阅成功回调
        """
        log_info('on_subscribe')
        if AdapayMessage.subscribe_callback is not None:
            AdapayMessage.subscribe_callback(0)

    @staticmethod
    def _on_unsubscribe(client, userdata, mid):
        """
        订阅成功回调
        """
        log_info('on_unsubscribe')
        if AdapayMessage.subscribe_callback is not None:
            AdapayMessage.subscribe_callback(-1)

    @staticmethod
    def _on_message(client, userdata, messages):
        """
        接收到交易结果回调
        """
        message_str = messages.payload.decode('utf-8')
        log_info('on_msg_receive:' + message_str)

        if AdapayMessage.message_received is not None:
            try:
                msg_dict = json.loads(message_str)
                AdapayMessage.message_received(msg_dict)
            except Exception as e:
                log_error(str(e))
                log_error('pay message loads error:' + message_str)

    @staticmethod
    def set_subscribe_callback(connect_callback=None, subscribe_callback=None):
        """
        设置建立长连接和订阅的回调
        :return:
        """
        AdapayMessage.connect_callback = connect_callback
        AdapayMessage.subscribe_callback = subscribe_callback

    @staticmethod
    def subscribe(on_message_received):
        AdapayMessage.message_received = on_message_received
        pay_msg_thread = threading.Thread(target=AdapayMessage._execute)
        pay_msg_thread.start()

    @staticmethod
    def _execute():
        client = mqtt.Client(AdapayMessage.client_id, protocol=mqtt.MQTTv311, clean_session=True)

        client.on_connect = AdapayMessage._on_connect
        client.on_disconnect = AdapayMessage._on_disconnect
        client.on_subscribe = AdapayMessage._on_subscribe
        client.on_unsubscribe = AdapayMessage._on_unsubscribe
        client.on_message = AdapayMessage._on_message

        user_name = 'Token|' + AdapayMessage.access_key + '|' + AdapayMessage.instance_id
        token = AdapayMessage._request_token()
        password = 'R|' + token
        client.username_pw_set(user_name, password)
        client.connect(AdapayMessage.broker_url, 1883, 60)
        client.loop_forever()

    @staticmethod
    def _request_token():
        expire_time = 30_000_000_000
        data = ApiRequest.post(adapay.base_url + pay_message_token, {'expire_time': expire_time})

        if 'succeeded' != data.get('status'):
            log_info('token request failed')

        return data.get('token', '')




"""
2019.8.1 create by jun.hu
参数处理
"""
import json


def pop_empty_value(params_dict):
    """
    去掉参数中value为空的字段
    :return: 去除空参数后的参数字典
    """
    temp_dict = dict()
    temp_dict.update(params_dict)
    for (key, value) in params_dict.items():
        if value is None or value == '' or value == {} or value == []:
            temp_dict.pop(key)
    return temp_dict


def get_plain_text(all_params):
    temp_list = list()
    for (k, v) in sorted(all_params.items()):
        if not isinstance(v, str):
            v = json.dumps(v, ensure_ascii=False)
        temp_list.append('{}={}'.format(str(k), str(v)))
    plain_text = '&'.join(temp_list)
    return plain_text


def read_file(file_path):
    with open(file_path, 'r') as f:
        return f.read()



import adapay
from adapay.api import urls
from adapay.api.api_request import ApiRequest


class Bill(object):

    @classmethod
    def download(cls, **kwargs):
        url = adapay.base_url + urls.bill_download
        return ApiRequest.post(url, kwargs)





import base64

from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5  # 加密解密模块
from Crypto.Hash import SHA1
from Crypto.PublicKey import RSA
from Crypto.Signature import pkcs1_15  # 加签验签模块


def rsa_long_encrypt(message, my_rsa_public):
    msg = message.encode('utf-8')  # 待加密信息转成utf-8形式
    length = len(msg)
    default_length = 245
    pubobj = Cipher_pkcs1_v1_5.new(RSA.importKey(my_rsa_public))
    if length < default_length:
        return base64.b64encode(pubobj.encrypt(msg))
    offset = 0
    res = []
    while length - offset > 0:
        if length - offset > default_length:
            res.append(pubobj.encrypt(msg[offset:offset + default_length]))
        else:
            res.append(pubobj.encrypt(msg[offset:]))
        offset += default_length
    byte_data = b''.join(res)
    return base64.b64encode(byte_data)  # 对返回内容以base64编码


def rsa_long_decrypt(message, my_rsa_private):
    msg = base64.b64decode(message)
    length = len(msg)
    default_length = 256
    priobj = Cipher_pkcs1_v1_5.new(RSA.importKey(my_rsa_private))
    if length < default_length:
        return b''.join(priobj.decrypt(msg, b'xyz'))
    offset = 0
    res = []
    while length - offset > 0:
        if length - offset > default_length:
            res.append(priobj.decrypt(msg[offset:offset + default_length], b'xyz'))
        else:
            res.append(priobj.decrypt(msg[offset:], b'xyz'))
        offset += default_length
        m = b''.join(res)
        n = m.decode("utf-8")
        print(n)
    return n


def rsa_sign(private_key, message, charset):
    try:
        private_key = add_start_end(private_key, "-----BEGIN PRIVATE KEY-----\n", "\n-----END PRIVATE KEY-----")
        msg = message.encode(charset)
        private_key = RSA.importKey(private_key)
        hash_obj = SHA1.new(msg)
        signature = pkcs1_15.new(private_key).sign(hash_obj)
        return True, base64.b64encode(signature).decode(charset)
    except Exception as ex:
        return False, str(ex)


def rsa_design(signature, message, my_rsa_public):
    my_rsa_public = fill_public_key_marker(my_rsa_public)
    message = message.encode("utf-8")
    public_key = RSA.importKey(my_rsa_public)
    hash_obj = SHA1.new(message)
    try:
        pkcs1_15.new(public_key).verify(hash_obj, base64.b64decode(signature))
        return True, ''
    except (ValueError, TypeError) as ex:
        return False, str(ex)


def add_start_end(key, start_marker, end_marker):
    if key.find(start_marker) < 0:
        key = start_marker + key
    if key.find(end_marker) < 0:
        key = key + end_marker
    return key


def fill_private_key_marker(private_key):
    return add_start_end(private_key, "-----BEGIN PRIVATE KEY-----\n", "\n-----END PRIVATE KEY-----")


def fill_public_key_marker(public_key):
    return add_start_end(public_key, "-----BEGIN PUBLIC KEY-----\n", "\n-----END PUBLIC KEY-----")
